/**
  ******************************************************************************
  * @file    rsc_table.c
  * @author  MCD Application Team
  * @brief   Ressource table
  *
  *   This file provides a default resource table requested by remote proc to
  *  load the elf file. It also allows to add debug trace using a shared buffer.
  *
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2017 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/** @addtogroup RSC_TABLE
  * @{
  */

/** @addtogroup stm32mp1xx_resource_table
  * @{
  */

/** @addtogroup STM32MP1xx_resource_table_Private_Includes
  * @{
  */

//#include "stm32mp1xx_hal.h"
#if defined(__ICCARM__)
#include <stddef.h> /* needed  for offsetof definition*/
#endif


/**
  * @}
  */

/** @addtogroup STM32MP1xx_resource_table_Private_TypesDefinitions
  * @{
  */


/**
  * @}
  */

/** @addtogroup STM32MP1xx_resource_table_Private_Defines
  * @{
  */
enum fw_resource_type {
	RSC_CARVEOUT = 0,
	RSC_DEVMEM = 1,
	RSC_TRACE = 2,
	RSC_VDEV = 3,
	RSC_LAST = 4,
};
/* Place resource table in special ELF section */
#if defined(__ICCARM__)
//#pragma section =".resource_table"
#elif defined(__GNUC__)
#define __section_t(S)          __attribute__((__section__(#S)))
#define __resource              __section_t(.resource_table)
#endif
struct fw_rsc_trace {
	unsigned int type;
	unsigned int da;
	unsigned int len;
	unsigned int reserved;
	unsigned char name[32];
} __packed;

/* Resource table for the given remote */
struct remote_resource_table {
	unsigned int version;
	unsigned int num;
	unsigned int reserved[2];
	unsigned int offset[1];
	struct fw_rsc_trace cm_trace;
};



#if defined (__LOG_TRACE_IO_)
extern char system_log_buf[];

#if defined(__ICCARM__)
__root struct remote_resource_table rproc_resource @ ".resource_table" = { \

	.version = 1, \
	.num = 1, \
	.reserved = {0, 0}, \
	.offset = { \
		offsetof(struct remote_resource_table, cm_trace), \
	}, \
	.cm_trace = {
		RSC_TRACE,
		(uint32_t)system_log_buf, SYSTEM_TRACE_BUF_SZ, 0, "cm4_log",
	},
};

#elif defined(__GNUC__)
struct remote_resource_table __resource __attribute__((used)) rproc_resource = { \

	.version = 1, \
	.num = 1, \
	.reserved = {0, 0}, \
	.offset = { \
		offsetof(struct remote_resource_table, cm_trace), \
	}, \
	.cm_trace = {
		RSC_TRACE,
		(uint32_t)system_log_buf, SYSTEM_TRACE_BUF_SZ, 0, "cm4_log",
	},
};
#endif

#else /* !__LOG_TRACE_IO_ */

#if defined(__ICCARM__)
__root struct remote_resource_table rproc_resource @ ".resource_table" = { \
	.version = 1,\
	.num = 0,\
	.reserved = {0, 0},\
	.offset = { 0 },\
};

#elif defined(__GNUC__)
struct remote_resource_table __resource __attribute__((used)) rproc_resource = { \
	.version = 1,\
	.num = 0,\
	.reserved = {0, 0},\
	.offset = { 0 },\
};
#endif


#endif /* __LOG_TRACE_IO_ */



